@@ -5,46 +5,47 @@ import (
 	"fmt"
 	"time"
 
-	"github.com/aws/amazon-ec2-instance-selector/v2/pkg/selector"
-
-	"github.com/weaveworks/eksctl/pkg/actions/nodegroup"
+	"k8s.io/client-go/kubernetes"
 
 	"github.com/kris-nova/logger"
 	"github.com/spf13/cobra"
 	"github.com/spf13/pflag"
 
+	"github.com/weaveworks/eksctl/pkg/actions/nodegroup"
 	api "github.com/weaveworks/eksctl/pkg/apis/eksctl.io/v1alpha5"
 	"github.com/weaveworks/eksctl/pkg/authconfigmap"
 	"github.com/weaveworks/eksctl/pkg/ctl/cmdutils"
 	"github.com/weaveworks/eksctl/pkg/ctl/cmdutils/filter"
 )
 
+type deleteNodeGroupOptions struct {
+	updateAuthConfigMap   *bool
+	deleteNodeGroupDrain  bool
+	onlyMissing           bool
+	maxGracePeriod        time.Duration
+	podEvictionWaitPeriod time.Duration
+	disableEviction       bool
+	parallel              int
+}
+
 func deleteNodeGroupCmd(cmd *cmdutils.Cmd) {
-	deleteNodeGroupWithRunFunc(cmd, func(cmd *cmdutils.Cmd, ng *api.NodeGroup, updateAuthConfigMap, deleteNodeGroupDrain, onlyMissing bool, maxGracePeriod, podEvictionWaitPeriod time.Duration, disableEviction bool, parallel int) error {
-		return doDeleteNodeGroup(cmd, ng, updateAuthConfigMap, deleteNodeGroupDrain, onlyMissing, maxGracePeriod, podEvictionWaitPeriod, disableEviction, parallel)
+	deleteNodeGroupWithRunFunc(cmd, func(cmd *cmdutils.Cmd, ng *api.NodeGroup, options deleteNodeGroupOptions) error {
+		return doDeleteNodeGroup(cmd, ng, options)
 	})
 }
 
-func deleteNodeGroupWithRunFunc(cmd *cmdutils.Cmd, runFunc func(cmd *cmdutils.Cmd, ng *api.NodeGroup, updateAuthConfigMap, deleteNodeGroupDrain, onlyMissing bool, maxGracePeriod, podEvictionWaitPeriod time.Duration, disableEviction bool, parallel int) error) {
+func deleteNodeGroupWithRunFunc(cmd *cmdutils.Cmd, runFunc func(cmd *cmdutils.Cmd, ng *api.NodeGroup, options deleteNodeGroupOptions) error) {
 	cfg := api.NewClusterConfig()
 	ng := api.NewNodeGroup()
 	cmd.ClusterConfig = cfg
 
-	var (
-		updateAuthConfigMap   bool
-		deleteNodeGroupDrain  bool
-		onlyMissing           bool
-		maxGracePeriod        time.Duration
-		podEvictionWaitPeriod time.Duration
-		disableEviction       bool
-		parallel              int
-	)
+	var options deleteNodeGroupOptions
 
 	cmd.SetDescription("nodegroup", "Delete a nodegroup", "", "ng")
 
 	cmd.CobraCommand.RunE = func(_ *cobra.Command, args []string) error {
 		cmd.NameArg = cmdutils.GetNameArg(args)
-		return runFunc(cmd, ng, updateAuthConfigMap, deleteNodeGroupDrain, onlyMissing, maxGracePeriod, podEvictionWaitPeriod, disableEviction, parallel)
+		return runFunc(cmd, ng, options)
 	}
 
 	cmd.FlagSetGroup.InFlagSet("General", func(fs *pflag.FlagSet) {
@@ -54,16 +55,15 @@ func deleteNodeGroupWithRunFunc(cmd *cmdutils.Cmd, runFunc func(cmd *cmdutils.Cm
 		cmdutils.AddConfigFileFlag(fs, &cmd.ClusterConfigFile)
 		cmdutils.AddApproveFlag(fs, cmd)
 		cmdutils.AddNodeGroupFilterFlags(fs, &cmd.Include, &cmd.Exclude)
-		fs.BoolVar(&onlyMissing, "only-missing", false, "Only delete nodegroups that are not defined in the given config file")
-		cmdutils.AddUpdateAuthConfigMap(fs, &updateAuthConfigMap, "Remove nodegroup IAM role from aws-auth configmap")
-		fs.BoolVar(&deleteNodeGroupDrain, "drain", true, "Drain and cordon all nodes in the nodegroup before deletion")
+		fs.BoolVar(&options.onlyMissing, "only-missing", false, "Only delete nodegroups that are not defined in the given config file")
+		options.updateAuthConfigMap = cmdutils.AddUpdateAuthConfigMap(fs, "Remove nodegroup IAM role from aws-auth configmap")
+		fs.BoolVar(&options.deleteNodeGroupDrain, "drain", true, "Drain and cordon all nodes in the nodegroup before deletion")
 		defaultMaxGracePeriod, _ := time.ParseDuration("10m")
-		fs.DurationVar(&maxGracePeriod, "max-grace-period", defaultMaxGracePeriod, "Maximum pods termination grace period")
+		fs.DurationVar(&options.maxGracePeriod, "max-grace-period", defaultMaxGracePeriod, "Maximum pods termination grace period")
 		defaultPodEvictionWaitPeriod, _ := time.ParseDuration("10s")
-		fs.DurationVar(&podEvictionWaitPeriod, "pod-eviction-wait-period", defaultPodEvictionWaitPeriod, "Duration to wait after failing to evict a pod")
-		defaultDisableEviction := false
-		fs.BoolVar(&disableEviction, "disable-eviction", defaultDisableEviction, "Force drain to use delete, even if eviction is supported. This will bypass checking PodDisruptionBudgets, use with caution.")
-		fs.IntVar(&parallel, "parallel", 1, "Number of nodes to drain in parallel. Max 25")
+		fs.DurationVar(&options.podEvictionWaitPeriod, "pod-eviction-wait-period", defaultPodEvictionWaitPeriod, "Duration to wait after failing to evict a pod")
+		fs.BoolVar(&options.disableEviction, "disable-eviction", false, "Force drain to use delete, even if eviction is supported. This will bypass checking PodDisruptionBudgets, use with caution.")
+		fs.IntVar(&options.parallel, "parallel", 1, "Number of nodes to drain in parallel. Max 25")
 
 		cmd.Wait = false
 		cmdutils.AddWaitFlag(fs, &cmd.Wait, "deletion of all resources")
@@ -73,7 +73,15 @@ func deleteNodeGroupWithRunFunc(cmd *cmdutils.Cmd, runFunc func(cmd *cmdutils.Cm
 	cmdutils.AddCommonFlagsForAWS(cmd, &cmd.ProviderConfig, true)
 }
 
-func doDeleteNodeGroup(cmd *cmdutils.Cmd, ng *api.NodeGroup, updateAuthConfigMap, deleteNodeGroupDrain, onlyMissing bool, maxGracePeriod time.Duration, podEvictionWaitPeriod time.Duration, disableEviction bool, parallel int) error {
+type authConfigMapUpdater struct {
+	clientSet kubernetes.Interface
+}
+
+func (a *authConfigMapUpdater) RemoveNodeGroup(ng *api.NodeGroup) error {
+	return authconfigmap.RemoveNodeGroup(a.clientSet, ng)
+}
+
+func doDeleteNodeGroup(cmd *cmdutils.Cmd, ng *api.NodeGroup, options deleteNodeGroupOptions) error {
 	ngFilter := filter.NewNodeGroupFilter()
 
 	if err := cmdutils.NewDeleteAndDrainNodeGroupLoader(cmd, ng, ngFilter).Load(); err != nil {
@@ -101,7 +109,7 @@ func doDeleteNodeGroup(cmd *cmdutils.Cmd, ng *api.NodeGroup, updateAuthConfigMap
 
 	if cmd.ClusterConfigFile != "" {
 		logger.Info("comparing %d nodegroups defined in the given config (%q) against remote state", len(cfg.NodeGroups), cmd.ClusterConfigFile)
-		if onlyMissing {
+		if options.onlyMissing {
 			err = ngFilter.SetOnlyRemote(ctx, ctl.AWSProvider.EKS(), stackManager, cfg)
 			if err != nil {
 				return err
@@ -118,7 +126,7 @@ func doDeleteNodeGroup(cmd *cmdutils.Cmd, ng *api.NodeGroup, updateAuthConfigMap
 
 	logFiltered()
 
-	if updateAuthConfigMap {
+	if api.IsEnabled(options.updateAuthConfigMap) {
 		for _, ng := range cfg.NodeGroups {
 			if ng.IAM == nil || ng.IAM.InstanceRoleARN == "" {
 				if err := ctl.GetNodeGroupIAM(ctx, stackManager, ng); err != nil {
@@ -130,26 +138,24 @@ func doDeleteNodeGroup(cmd *cmdutils.Cmd, ng *api.NodeGroup, updateAuthConfigMap
 	}
 	allNodeGroups := cmdutils.ToKubeNodeGroups(cfg)
 
-	instanceSelector, err := selector.New(ctx, ctl.AWSProvider.AWSConfig())
-	if err != nil {
-		return err
-	}
-	nodeGroupManager := nodegroup.New(cfg, ctl, clientSet, instanceSelector)
-	if deleteNodeGroupDrain {
+	if options.deleteNodeGroupDrain {
 		cmdutils.LogIntendedAction(cmd.Plan, "drain %d nodegroup(s) in cluster %q", len(allNodeGroups), cfg.Metadata.Name)
 
 		drainInput := &nodegroup.DrainInput{
 			NodeGroups:            allNodeGroups,
 			Plan:                  cmd.Plan,
-			MaxGracePeriod:        maxGracePeriod,
-			PodEvictionWaitPeriod: podEvictionWaitPeriod,
-			DisableEviction:       disableEviction,
-			Parallel:              parallel,
+			MaxGracePeriod:        options.maxGracePeriod,
+			PodEvictionWaitPeriod: options.podEvictionWaitPeriod,
+			DisableEviction:       options.disableEviction,
+			Parallel:              options.parallel,
 		}
-		ctx, cancel := context.WithTimeout(ctx, cmd.ProviderConfig.WaitTimeout)
+		drainCtx, cancel := context.WithTimeout(ctx, cmd.ProviderConfig.WaitTimeout)
 		defer cancel()
-		err := nodeGroupManager.Drain(ctx, drainInput)
-		if err != nil {
+
+		drainer := &nodegroup.Drainer{
+			ClientSet: clientSet,
+		}
+		if err := drainer.Drain(drainCtx, drainInput); err != nil {
 			logger.Warning("error occurred during drain, to skip drain use '--drain=false' flag")
 			return err
 		}
@@ -157,27 +163,23 @@ func doDeleteNodeGroup(cmd *cmdutils.Cmd, ng *api.NodeGroup, updateAuthConfigMap
 
 	cmdutils.LogIntendedAction(cmd.Plan, "delete %d nodegroups from cluster %q", len(allNodeGroups), cfg.Metadata.Name)
 
-	err = nodeGroupManager.Delete(ctx, cfg.NodeGroups, cfg.ManagedNodeGroups, cmd.Wait, cmd.Plan)
-	if err != nil {
+	deleter := &nodegroup.Deleter{
+		StackHelper:      ctl.NewStackManager(cfg),
+		NodeGroupDeleter: ctl.AWSProvider.EKS(),
+		ClusterName:      cfg.Metadata.Name,
+		AuthConfigMapUpdater: &authConfigMapUpdater{
+			clientSet: clientSet,
+		},
+	}
+	if err := deleter.Delete(ctx, cfg.NodeGroups, cfg.ManagedNodeGroups, nodegroup.DeleteOptions{
+		Wait:                cmd.Wait,
+		Plan:                cmd.Plan,
+		UpdateAuthConfigMap: !api.IsDisabled(options.updateAuthConfigMap),
+	}); err != nil {
 		return err
 	}
 
-	if updateAuthConfigMap {
-		cmdutils.LogIntendedAction(cmd.Plan, "delete %d nodegroups from auth ConfigMap in cluster %q", len(cfg.NodeGroups), cfg.Metadata.Name)
-		if !cmd.Plan {
-			for _, ng := range cfg.NodeGroups {
-				if ng.IAM != nil && ng.IAM.InstanceRoleARN != "" {
-					if err := authconfigmap.RemoveNodeGroup(clientSet, ng); err != nil {
-						logger.Warning(err.Error())
-					}
-				}
-			}
-		}
-	}
-
 	cmdutils.LogCompletedAction(cmd.Plan, "deleted %d nodegroup(s) from cluster %q", len(allNodeGroups), cfg.Metadata.Name)
-
 	cmdutils.LogPlanModeWarning(cmd.Plan && len(allNodeGroups) > 0)
-
 	return nil
 }
